/*****************************************************************************************

Utilities.c - utility routines for NTBDemo

Written by Bryan K. Ressler (Beaker), 8/30/91

updated for the wonderful universe of PowerPC by Pete Gontier (Gurgle), 3/31/97

*****************************************************************************************/

/** INCLUDES ****************************************************************************/
#include "NTBDemo.h"			/* NTBDemo stuff */
#include "NeoTextBox.h"
#include "Utilities.h"

#ifndef __TEXTEDIT__
#	include <TextEdit.h>
#endif

#ifndef __FONTS__
#	include <Fonts.h>
#endif

/****************************************************************************************/
short JustToRadio(short justification)
{
	short result = kJustFull;

	switch (justification)
	{
		case teJustLeft:
			result = kJustLeft;
			break;
		case teJustCenter:
			result = kJustCenter;
			break;
		case teJustRight:
			result = kJustRight;
			break;
		case ntbJustFull:
			result = kJustFull;
			break;
	}

	return result;
}

/****************************************************************************************/
short RadioToJust(void)
{
	static short justifications[4] = { teJustLeft,teJustCenter,teJustRight,ntbJustFull };
	return (justifications[gJustRadio - kJustLeft]);
}

/****************************************************************************************/
short RadioToFont(void)
{
	short familyID = GetAppFont();
	
	switch(gFontRadio) {
		case kAppFont:
			familyID = GetAppFont();
			break;
		case kTimes:
			GetFNum("\pTimes",&familyID);			/* Zero (sys font) if not present */
			break;
		case kHelvetica:
			GetFNum("\pHelvetica",&familyID);		/* Zero (sys font) if not present */
			break;
	}

	return familyID;
}

/****************************************************************************************/
void SetValue(DialogPtr theDialog,short itemNum,short value)
{
	short	aType;
	Rect	aBox;
	Handle	theItem;

	GetDialogItem(theDialog,itemNum,&aType,&theItem,&aBox);
	SetControlValue((ControlHandle)theItem,value);
}

/****************************************************************************************/
void GetItemRect(DialogPtr theDialog,short itemNum,Rect *box)
{
	short	aType;
	Handle	theItem;

	GetDialogItem(theDialog,itemNum,&aType,&theItem,box);
}

/****************************************************************************************/
void UserItem (DialogPtr theDialog, short itemNum, UserItemProcPtr theProc)
{
			short				aType;
			Rect				aBox;
			Handle				theItem;

	//
	//	This is a cheesy hack. This function was not designed
	//	to produce errors and I didn't feel like revising the
	//	callers. So we "cache" the UPP because we know the
	//	sequence in which we will be called. -- PG
	//

	static	UserItemProcPtr		userItemProcPtr;
	static	UserItemUPP			userItemUPP;

	if (userItemProcPtr != theProc)
	{
		userItemUPP = NewUserItemProc (theProc);
		userItemProcPtr = theProc;
	}

	GetDialogItem(theDialog,itemNum,&aType,&theItem,&aBox);
	SetDialogItem(theDialog,itemNum,aType,(Handle)userItemUPP,&aBox);
}

/****************************************************************************************/
void InvalItem(DialogPtr theDialog,short itemNum,short hInset,short vInset)
{
	short	aType;
	Rect	aBox;
	Handle	theItem;

	GetDialogItem(theDialog,itemNum,&aType,&theItem,&aBox);
	InsetRect(&aBox,hInset,vInset);
	InvalRect(&aBox);
}

/****************************************************************************************/
pascal void BoxItem(WindowPtr theWindow,short itemNum)
{
	short	aType;
	Rect	theBox;
	Handle	aHandle;

	GetDialogItem(theWindow,itemNum,&aType,&aHandle,&theBox);
	PenNormal();
	ForeColor(blackColor);
	
	FrameRect(&theBox);
}

/****************************************************************************************/
pascal void GrayBoxItem(WindowPtr theWindow,short itemNum)
{
	short	aType;
	Rect	theBox;
	Handle	aHandle;

	GetDialogItem(theWindow,itemNum,&aType,&aHandle,&theBox);
	PenNormal(); PenPat(&qd.gray);
	ForeColor(blackColor);
	
	FrameRect(&theBox);
	PenNormal();
}

/****************************************************************************************/
void TextParms(short code)
{
	static short txFont,txFace,txSize,txMode;
	
	if (code == kSave) {		/* Save current port's text parameters */
		txFont = qd.thePort->txFont;
		txFace = qd.thePort->txFace;
		txSize = qd.thePort->txSize;
		txMode = qd.thePort->txMode;
	} else {					/* Restore current port's text parameters */
		TextFont(txFont);
		TextFace(txFace);
		TextSize(txSize);
		TextMode(txMode);
	}
}
